unit EventLog;

interface

uses
  Classes, SysUtils, Forms;

type
  TEventType = (evtError,evtWarning,evtInformation,evtAuditSuccess,evtAuditFailure);
  TEventLog = class(TComponent)
  private
  	FEnabled: Boolean;
    FApplicationName: string;
    FRegisterApplication: Boolean;
    FIncludeUser: Boolean;
    FAutoLog: Boolean;
    FInitializationID: LongWord;
    FFinalizationID: LongWord;
    function GetName: string;
    procedure SetName(const Name: string);
  protected
  	procedure Loaded; override;
  public
    procedure Add(const ID: LongWord;
                  const AType: TEventType = evtInformation;
                  const Category: Word = 0;
                        Description: string = '');
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Name: string read GetName write SetName;
    property Enabled: Boolean read FEnabled write FEnabled;
    property ApplicationName: string read FApplicationName write FApplicationName;
    property RegisterApplication: Boolean read FRegisterApplication write FRegisterApplication;
    property IncludeUser: Boolean read FIncludeUser write FIncludeUser;
    property AutoLog: Boolean read FAutoLog write FAutoLog;
    property InitializationID: LongWord read FInitializationID write FInitializationID;
    property FinalizationID: LongWord read FFinalizationID write FFinalizationID;
  end;
  EEventType = class(Exception);
  EReportFail = class(EEventType);
  ERegisterSourceFail = class(EEventType);
  EUserNameLookupFail = class(EEventType);
  EUserSIDLookupFail = class(EEventType);
  ERegisterApplicationFail = class(EEventType);

var
  ApplicationEventLog: TEventLog;

procedure Register;

implementation

uses
  Windows, Registry;

const
  WordEventType: array [TEventType] of Word = (EVENTLOG_ERROR_TYPE,
                                               EVENTLOG_WARNING_TYPE,
                                               EVENTLOG_INFORMATION_TYPE,
                                               EVENTLOG_AUDIT_SUCCESS,
                                               EVENTLOG_AUDIT_FAILURE);
  RegistrationPath = '\SYSTEM\CurrentControlSet\Services\EventLog\Application';
  NewLineChars = Chr($0D) + Chr($0A);

resourcestring
  EReportFailMsg = ' - Event report fail, error code: ';
  ERegisterSourceFailMsg = ' - Event source registration fail, error code: ';
  EUserNameLookupFailMsg = ' - User SID lookup fail, error code: ';
  EUserSIDLookupFailMsg = ' - User SID lookup fail, error code: ';
  ERegisterApplicationFailMsg = ' - Application registration fail, error code: ';

procedure Register;
begin

  RegisterComponents('Not Visible', [TEventLog]);

end;

procedure TEventLog.Loaded;
begin

	inherited Loaded;
	if not (csDesigning in ComponentState) then begin
  	if FApplicationName = '' then
      FApplicationName := Application.Title;
		if FAutoLog then
  		Add(FInitializationID);
  end;

end;

function TEventLog.GetName: string;
begin

  if Self = ApplicationEventLog then
    inherited Name := 'ApplicationEventLog';
  Result := inherited Name;

end;

procedure TEventLog.SetName(const Name: string);
begin

  if Self = ApplicationEventLog then
    inherited Name := 'ApplicationEventLog'
  else
	  inherited Name := Name;

end;

var
	E: LongWord;

procedure TEventLog.Add(const ID: LongWord;
                        const AType: TEventType;
                        const Category: Word;
                              Description: string);

var
  EventSourceHandle: THandle;
  UserName: PChar;
  UserNameSize: LongWord;
  SID: Pointer;
  SIDSize: LongWord; 
  Domain: PChar;
  DomainSize: LongWord;
  SIDUse: SID_NAME_USE;
  DescriptionP: Pointer;
  LineN: Word;

begin

	if FEnabled then begin
    if FRegisterApplication and (FApplicationName <> '') then begin
      with TRegistry.Create do try
        RootKey := HKEY_LOCAL_MACHINE;
        if not OpenKey( RegistrationPath + '\' + FApplicationName,True) then
          raise ERegisterApplicationFail.Create(Self.Classname + ERegisterApplicationFailMsg + IntToStr(GetLastError));
      finally
        CloseKey;
        Free;
      end;
    end;
    EventSourceHandle := RegisterEventSource(nil,PChar(FApplicationName));
    UserName := nil;
    SID := nil;
    Domain := nil;
    try
      if EventSourceHandle = Null then
        raise ERegisterSourceFail.Create(Self.Classname + ERegisterSourceFailMsg + IntToStr(GetLastError));
      if Description <> '' then begin
        Description := NewLineChars + NewLineChars + Description;
        DescriptionP := PChar(Description);
        LineN := 1;
      end
      else
        LineN := 0;
      if FIncludeUser then begin
				UserNameSize := 64;
       	ReallocMem(UserName,UserNameSize + 1);
        while not GetUserName(UserName,UserNameSize) do begin
          if GetLastError <> ERROR_INSUFFICIENT_BUFFER then
      	    raise EUserNameLookupFail.Create(Self.Classname + EUserNameLookupFailMsg + IntToStr(GetLastError));
					UserNameSize := UserNameSize + 64;
	       	ReallocMem(UserName,UserNameSize + 1);
        end;
        SIDSize := 64;
        DomainSize := 64;
        ReallocMem(SID,SIDSize);
        ReallocMem(Domain,DomainSize);
        while not LookupAccountName(nil,UserName,SID,SIDSize,Domain,DomainSize,SIDUse) do begin
          if GetLastError <> ERROR_INSUFFICIENT_BUFFER then
	          raise EUserSIDLookupFail.Create(Self.Classname + EUserSIDLookupFailMsg + IntToStr(GetLastError));
        	ReallocMem(SID,SIDSize);
        	ReallocMem(Domain,DomainSize);
        end;
        if not ReportEvent(EventSourceHandle,WordEventType[AType],Category,ID,SID,LineN,0,@DescriptionP,nil) then
          raise EReportFail.Create(Self.Classname + EReportFailMsg + IntToStr(GetLastError));
      end
      else if not ReportEvent(EventSourceHandle,WordEventType[AType],Category,ID,nil,LineN,0,@DescriptionP,nil) then
        raise EReportFail.Create(Self.Classname + EReportFailMsg + IntToStr(GetLastError));
    finally
     	ReallocMem(Domain,0);
     	ReallocMem(SID,0);
     	ReallocMem(UserName,0);
      if EventSourceHandle <> Null then
        DeregisterEventSource(EventSourceHandle);
    end;
  end;

end;

constructor TEventLog.Create(AOwner: TComponent);
begin

  inherited Create(AOwner);
  FEnabled := true;
  FApplicationName := '';
  FRegisterApplication := true;
  FIncludeUser := true;
  FAutoLog := false;
  FInitializationID := 0;
  FFinalizationID := 0;
  if not Assigned(ApplicationEventLog) then
    ApplicationEventLog := Self;

end;

destructor TEventLog.Destroy;
begin

	if not (csDesigning in ComponentState) and FAutoLog then
  	Add(FFinalizationID);
	if ApplicationEventLog = Self then
		ApplicationEventLog := nil;
  inherited Destroy;

end;

initialization

  ApplicationEventLog := nil;

end.

